﻿using BPA.SAAS.Manage.Application.Org.Dtos.Role;
using BPA.SAAS.Manage.Application.Org.Interface;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Core.Org;
using BPA.SAAS.Manage.Core.System;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.Org.Services
{
    public class RoleService: IRoleService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public RoleService(ISqlSugarClient db) 
        {
            _db=db;
        }
        public async Task<bool> Add(RoleDtoInput input)
        {

            if (input.Code != "sys_manager_role")
            {
                var resEntitites = _db.Queryable<BPA_Roles>().Where(a => a.Code == input.Code).ToList();
                if (resEntitites.Count() > 0)
                {
                    throw Oops.Oh("已存在相同编码的角色");
                }
            }
            var role = input.Adapt<BPA_Roles>();
           // role.DataScopeType = DataScopeType.ALL;
            return await _db.Insertable(role).CallEntityMethod(t => t.Create()).ExecuteCommandAsync() > 0;
        }
        /// <summary>
        /// 根据角色查询角色菜单
        /// </summary>
        /// <param name="RoleId"></param>
        /// <returns></returns>
        public async Task<List<QueryRoleMenuListDto>> GetMenuRole(string RoleId)
        {
            var MenuName =await _db.Queryable<BPA_RoleMenu>().LeftJoin<BPA_Menu>((t, x) => t.SysMenuId == x.Id)
                  .Where(t => t.SysRoleId == RoleId).OrderBy((t, x) => x.CreateAt).Select((t, x) => new QueryRoleMenuListDto()
                  {
                      Name = x.Name,
                      Id = x.Id,
                      IsParentMenu = string.IsNullOrWhiteSpace(x.Pid)
                  }).ToListAsync();

            return MenuName;
        }
        /// <summary>
        /// 编辑角色菜单
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> AddOrUpdateMenuRole(RoleMenuDtoInput input)
        {
            List<BPA_RoleMenu> models = new List<BPA_RoleMenu>();
            List<string> NewMenuId = new List<string>();
            if (input.SysMenuId != null)
            {
                var Menus = _db.Queryable<BPA_Menu>().ToList();

                foreach (var item in input.SysMenuId)
                {
                    var Pid = Menus.FirstOrDefault(t => t.Id == item).Pid;
                    if (Pid != null && Pid != "")
                    {
                        var Pid2 = Menus.FirstOrDefault(t => t.Id == Pid).Pid;

                        if (Pid2 != null)
                        {
                            NewMenuId.Add(Pid2);
                        }
                        NewMenuId.Add(Pid);
                    }
                    NewMenuId.Add(item);
                }
                NewMenuId.Distinct().ToList().ForEach(item =>
                {
                    models.Add(new BPA_RoleMenu
                    {
                        Id = Guid.NewGuid().ToString(),
                        SysMenuId = item,
                        SysRoleId = input.SysRoleId
                    });
                });
            }
            if (models.Count <= 0)
                return true;

            _db.Deleteable<BPA_RoleMenu>(t => t.SysRoleId == input.SysRoleId).ExecuteCommandHasChange();
            return await _db.Insertable(models).ExecuteCommandAsync() > 0;


        }
    }
}
